import multiprocessing as mp
import os
import time


def f(f2):
    for i in range(10):
        time.sleep(0.1)
        f2()

def f3(f):
    f()


class A:
    def __init__(self):
        self.mac = os.urandom(6).hex()
        self.p = mp.Process(target=f, args=(self.f2,))
        self.p.start()

    def f(self):
        print(f'f is {self.mac}')

    def f2(self):
        print(f'f2 is {self.mac}')
        self.p = mp.Process(target=f3, args=(self.f,))
        self.p.start()

class B:
    def action1(self):
        ...

if __name__ == '__main__':
    a = B()
    res = hasattr(a, 'action1')
    print(res)
    print(dir(a))



import os

import pytest

from easysmart import Manager


def test_control():

    def on_connect(client, userdata, flags, rc):
        print('Connected with result code ' + str(rc))
        # client.subscribe(topic, 0)

    curPath = os.path.abspath(os.path.dirname(__file__))
    print(curPath)
    m = Manager()



import os, sys, pathlib
import time
from pathlib import Path


def start_emqx_server():
    print(f'system is {sys.platform}')
    if sys.platform == 'win32':
        pass
    else:
        raise NotImplementedError('only support windows now')

    # check emqx server
    if check_emqx_server():
        return
    print(f'starting emqx server')
    # if path not exists, create it
    emqx_path = Path(r'emqx\bin\emqx')
    # start emqx by run "emqx\bin\emqx start"
    os.system(f"{emqx_path} start")
    print('emqx server started')


def check_emqx_server():
    print(f'check emqx server')
    # if path not exists, create it
    emqx_path = Path(r'emqx\bin\emqx_ctl')
    # 检测emqx是否正在运行
    # run "emqx\bin\emqx status" and get the result
    result = os.popen(f"{emqx_path} status").read()
    print(f'emqx status is {result}')
    if 'is starting' in result:
        print('emqx server is starting check again after 3 seconds')
        time.sleep(3)
        return check_emqx_server()
    if 'is started' in result:
        print('emqx server is running')
        return True
    else:
        print('emqx server is not running')
        return False


def test_emqx_server():
    start_emqx_server()

if __name__ == '__main__':
    test_emqx_server()



import requests
import json
import os
import pathlib
import logging

DEVICE_DEFINE_PATH = r'script\device\devices_define'


def get_device_define(device_type):
    defines = []
    device_define = _get_device_define(device_type)
    defines.append(device_define)
    while device_define and device_define.get('parent'):
        device_define = _get_device_define(device_define['parent'])
        defines.append(device_define)
    defines.reverse()
    for define in defines:
        if define is None: continue
        device_define['properties'].update(define['properties'])
        device_define['actions'].update(define['actions'])
        for k,v in define.items():
            if k in ['properties','actions']: continue
            device_define[k] = v
    return device_define


def _get_device_define(device_type):
    logging.info(f'get device define {device_type}')
    # check if dir devices_define exists
    if not os.path.exists(DEVICE_DEFINE_PATH):
        logging.info(f'create devices_define dir')
        os.makedirs(DEVICE_DEFINE_PATH)
    json_file = f'{DEVICE_DEFINE_PATH}/{device_type}.json'
    if os.path.exists(json_file):
        logging.info(f'get device define from local')
        with open(json_file, 'r', encoding='utf-8') as f:
            device_define = json.load(f)
            return device_define
    else:
        # download device define from server and save it to local
        # url: http://device.easysmart.top/device_define/{device_type}.json
        logging.info(f'get device define from server')
        url = f'http://device.easysmart.top/device_define/{device_type}.json'
        try:
            r = requests.get(url)
            if r.status_code != 200:
                logging.warning(f'get device define {device_type} failed {r.status_code}')
                return
            device_define = r.json()
        except Exception as e:
            logging.warning(f'get device define {device_type} failed {e}')
            return
        with open(f'{DEVICE_DEFINE_PATH}/{device_type}.json', 'w', encoding='utf-8') as f:
            json.dump(device_define, f)
        return device_define


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    get_device_define('QTZ01')
    # get_device_define('base_device')



import os

import setuptools
import time
import datetime

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

EASYSMART_VERSION = '20230911215036'
time_version = EASYSMART_VERSION
# time_version = datetime.datetime.now().strftime('%Y%m%d')

setuptools.setup(
    # 包的分发名称，使用字母、数字、_、-
    name="easysmart",
    # 版本号, 版本号规范：https://www.python.org/dev/peps/pep-0440/
    # version="0.0.5",
    version=time_version,
    # 作者名
    author="jiandanzhineng",
    # 作者邮箱
    author_email="jiandanzhineng@outlook.com",
    # 包的简介描述
    description="简单智能官方实现",
    # 包的详细介绍(一般通过加载README.md)
    long_description=long_description,
    # 和上条命令配合使用，声明加载的是markdown文件
    long_description_content_type="text/markdown",
    # 项目开源地址
    url="http://easysmart.top",
    # 如果项目由多个文件组成，我们可以使用find_packages()自动发现所有包和子包，而不是手动列出每个包，在这种情况下，包列表将是example_pkg
    packages=setuptools.find_packages(),
    install_requires=['paho-mqtt', 'zeroconf', 'requests'],
    # 关于包的其他元数据(metadata)
    classifiers=[
        # 该软件包仅与Python3兼容
        "Programming Language :: Python :: 3",
        # 根据MIT许可证开源
        "License :: OSI Approved :: MIT License",
        # 与操作系统无关
        "Operating System :: Microsoft :: Windows",
    ],
)



import json
import logging
import time

from .device_define import get_device_define


class BaseDevice:

    def __init__(self, mac, device_type, mqtt_client):
        self.device_type = device_type
        self.mac = mac
        self.last_active_time = time.time()
        self.msg_id = 0
        self.mqtt_client = mqtt_client

        self.publish_topic = f'/drecv/{mac}'

        device_define = get_device_define(device_type)

        if device_define is None:
            raise Exception(f'get_device_define error: {device_type}')
        self.properties = device_define['properties']
        self.actions = device_define['actions']
        self.name = device_define['name']



    def publish(self, data):
        data['msg_id'] = self.msg_id
        self.msg_id += 1
        payload = json.dumps(data)
        self.mqtt_client.publish(self.publish_topic, payload)

    def get_property(self, property_name):
        data = {
            'method': 'get',
            'key': property_name,
        }
        self.publish(data)

    def set_property(self, property_name, value):
        if self.properties[property_name]['type'] == 'int':
            value = int(value)
        elif self.properties[property_name]['type'] == 'float':
            value = float(value)
        data = {
            'method': 'set',
            'key': property_name,
            'value': value,
        }
        self.publish(data)

    def execute_action(self, action_name):
        ...

    def update(self, property_name, value):
        update_flag = False
        if property_name not in self.properties:
            logging.warning(f'property {property_name} not in {self.mac}')
            return
        if not hasattr(self, property_name):
            # 如果没有这个属性，就创建一个
            update_flag = True
            setattr(self, property_name, value)
        else:
            # 如果有这个属性，就更新这个属性
            old_value = getattr(self, property_name)
            if old_value != value:
                update_flag = True
                setattr(self, property_name, value)
        if update_flag:
            print(f'update {self.mac}:{property_name}->{value}')
        self.last_active_time = time.time()
